"use strict";
var NativeAuthGuard_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.NativeAuthGuard = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const sdk_nestjs_cache_1 = require("@multiversx/sdk-nestjs-cache");
const sdk_native_auth_server_1 = require("@multiversx/sdk-native-auth-server");
const sdk_nestjs_common_1 = require("@multiversx/sdk-nestjs-common");
const sdk_nestjs_monitoring_1 = require("@multiversx/sdk-nestjs-monitoring");
const native_auth_invalid_origin_error_1 = require("./errors/native.auth.invalid.origin.error");
const decorators_1 = require("./decorators");
let NativeAuthGuard = NativeAuthGuard_1 = class NativeAuthGuard {
    constructor(mxnestConfigService, cacheService) {
        const nativeAuthServerConfig = {
            apiUrl: mxnestConfigService.getApiUrl(),
            maxExpirySeconds: mxnestConfigService.getNativeAuthMaxExpirySeconds(),
            acceptedOrigins: mxnestConfigService.getNativeAuthAcceptedOrigins(),
            cache: {
                getValue: async function (key) {
                    if (key === 'block:timestamp:latest') {
                        return new Date().getTime() / 1000;
                    }
                    if (cacheService) {
                        return await cacheService.get(key);
                    }
                    return undefined;
                },
                setValue: async function (key, value, ttl) {
                    if (cacheService) {
                        return await cacheService.set(key, value, ttl);
                    }
                    return undefined;
                },
            },
        };
        const acceptedOrigins = mxnestConfigService.getNativeAuthAcceptedOrigins();
        const shouldAllowAllOrigins = acceptedOrigins && acceptedOrigins.length === 1 && acceptedOrigins[0] === '*';
        if (shouldAllowAllOrigins) {
            nativeAuthServerConfig.isOriginAccepted = () => true;
        }
        this.authServer = new sdk_native_auth_server_1.NativeAuthServer(nativeAuthServerConfig);
    }
    static getOrigin(headers) {
        const origin = headers['origin'];
        if (origin) {
            return origin;
        }
        const referer = headers['referer'];
        if (referer) {
            try {
                const url = new URL(referer);
                return `${url.protocol}//${url.host}`;
            }
            catch (error) {
                const logger = new common_1.Logger(NativeAuthGuard_1.name);
                logger.error(`Could not parse referer '${referer}' into an URL`);
                logger.error(error);
            }
        }
        return '';
    }
    async canActivate(context) {
        const noAuthMetadata = sdk_nestjs_common_1.DecoratorUtils.getMethodDecorator(decorators_1.NoAuthOptions, context.getHandler());
        if (noAuthMetadata) {
            return true;
        }
        const request = sdk_nestjs_common_1.ExecutionContextUtils.getRequest(context);
        const response = sdk_nestjs_common_1.ExecutionContextUtils.getResponse(context);
        const headers = sdk_nestjs_common_1.ExecutionContextUtils.getHeaders(context);
        const origin = NativeAuthGuard_1.getOrigin(headers);
        const authorization = headers['authorization'];
        if (!authorization) {
            return false;
        }
        const jwt = authorization.replace('Bearer ', '');
        const profiler = new sdk_nestjs_monitoring_1.PerformanceProfiler();
        try {
            const userInfo = await this.authServer.validate(jwt);
            profiler.stop();
            if (!sdk_nestjs_common_1.UrlUtils.isLocalhost(origin) && origin !== userInfo.origin && origin !== 'https://' + userInfo.origin) {
                throw new native_auth_invalid_origin_error_1.NativeAuthInvalidOriginError(userInfo.origin, origin);
            }
            if (response) {
                response.set('X-Native-Auth-Issued', userInfo.issued);
                response.set('X-Native-Auth-Expires', userInfo.expires);
                response.set('X-Native-Auth-Address', userInfo.address);
                response.set('X-Native-Auth-Timestamp', Math.round(new Date().getTime() / 1000));
                response.set('X-Native-Auth-Duration', profiler.duration);
            }
            if (request) {
                request.nativeAuth = userInfo;
                request.jwt = userInfo;
            }
            return true;
        }
        catch (error) {
            if (error instanceof sdk_native_auth_server_1.NativeAuthError) {
                const message = error === null || error === void 0 ? void 0 : error.message;
                if (message) {
                    profiler.stop();
                    request.res.set('X-Native-Auth-Error-Type', error.constructor.name);
                    request.res.set('X-Native-Auth-Error-Message', message);
                    request.res.set('X-Native-Auth-Duration', profiler.duration);
                }
                return false;
            }
            throw error;
        }
    }
};
NativeAuthGuard = NativeAuthGuard_1 = tslib_1.__decorate([
    (0, common_1.Injectable)(),
    tslib_1.__param(0, (0, common_1.Inject)(sdk_nestjs_common_1.MXNEST_CONFIG_SERVICE)),
    tslib_1.__param(1, (0, common_1.Optional)()),
    tslib_1.__metadata("design:paramtypes", [Object, sdk_nestjs_cache_1.CacheService])
], NativeAuthGuard);
exports.NativeAuthGuard = NativeAuthGuard;
//# sourceMappingURL=native.auth.guard.js.map